/* language.c
 * Copyright (C) 2007-2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of the linutop config editor
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "main.h"
#include "utils.h"

GtkTreeIter
populate_locale_tree(GtkTreeStore * treestore)
{
	int i = 0;
    gboolean found = FALSE;
    GtkTreeIter iter;
    GtkTreeIter ret;
    gchar *code = g_strdup (get_locale_value ());

	while (locale_data[i].code != NULL) {
		gtk_tree_store_append(treestore, &iter, NULL);
		gtk_tree_store_set(treestore, &iter,
				   0, locale_data[i].name, 1, locale_data[i].code, -1);
        if (strcmp (locale_data[i].code, code) == 0) {
            ret = iter;
            found = TRUE;
        }
		i++;
	}
    g_free (code);

    if (!found)
        gtk_tree_model_get_iter_first (GTK_TREE_MODEL (treestore), &ret);
    return ret;
}

/* Dialog code */
void locale_dialog_run()
{
	GtkWidget *new_dialog, *scw1, *hbox;
	GtkWidget *tree_view = gtk_tree_view_new();
	GtkTreeStore *treestore;
	GtkCellRenderer *renderer;
	GtkTreeSelection *selection;
	GtkTreeIter iter, default_iter;
	GtkTreePath *default_iter_path;
	GtkTreeModel *model;

	new_dialog =
	    xfce_titled_dialog_new_with_buttons(_("Linutop - Language"),
						GTK_WINDOW(dialog->dialog),
						GTK_DIALOG_MODAL |
						GTK_DIALOG_DESTROY_WITH_PARENT,
						GTK_STOCK_CANCEL,
						GTK_RESPONSE_CANCEL,
						GTK_STOCK_OK, GTK_RESPONSE_OK,
						NULL);
	gtk_window_set_icon_name(GTK_WINDOW(new_dialog),
				 "preferences-desktop-locale");
	xfce_titled_dialog_set_subtitle((XfceTitledDialog *) new_dialog,
					_("Select a language"));

	hbox = gtk_hbox_new(TRUE, 6);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(new_dialog)->vbox), hbox);
	gtk_widget_show(hbox);

	scw1 = gtk_scrolled_window_new(NULL, NULL);
	gtk_box_pack_start(GTK_BOX(hbox), scw1, TRUE, TRUE, 2);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scw1),
				       GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_AUTOMATIC);
	gtk_widget_show(scw1);

	/* tree */
	treestore = gtk_tree_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
	default_iter = populate_locale_tree(GTK_TREE_STORE(treestore));
	renderer = gtk_cell_renderer_text_new();

	gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tree_view),
						    -1,
						    "Language",
						    renderer, "text", 0, NULL);
	gtk_tree_view_set_model(GTK_TREE_VIEW(tree_view),
				GTK_TREE_MODEL(treestore));
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree_view), FALSE);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));

	/* select default iter */
	gtk_tree_selection_select_iter(selection, &default_iter);

	/* scroll to the default iter */
	default_iter_path =
	    gtk_tree_model_get_path(gtk_tree_view_get_model
				    (GTK_TREE_VIEW(tree_view)), &default_iter);
	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(tree_view),
				     default_iter_path, NULL, TRUE, 0.5, 0.5);

	gtk_tree_path_free(default_iter_path);

	gtk_container_add(GTK_CONTAINER(scw1), tree_view);

	gtk_widget_show(tree_view);

	gtk_window_set_default_size(GTK_WINDOW(new_dialog), 400, 400);

	g_signal_connect_swapped(G_OBJECT(tree_view), "row-activated",
			 G_CALLBACK(cb_row_activated), new_dialog);

	int response = gtk_dialog_run(GTK_DIALOG(new_dialog));

	if (response == GTK_RESPONSE_OK) {
		gchar *locale;
		selection =
		    gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
		gtk_tree_selection_get_selected(selection, &model, &iter);
		gtk_tree_model_get(model, &iter, 1, &locale, -1);
		set_locale (locale);
        update_locale_settings (locale);
        g_free (locale);
	}
	gtk_widget_destroy(new_dialog);
	return;
}
