/* pwd.c
 * Copyright (C) 2007-2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of the linutop config editor
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define _XOPEN_SOURCE
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "main.h"

enum {
	OK = 0,
	CANCEL
};

typedef struct _p {
	GtkWidget *entry1;
	GtkWidget *entry2;
	GtkWidget *okbtn;
	gchar *pwd;
} PW;

gchar *get_hash(const gchar * clear)
{
	const char s[] =
	    "azertyuiopmlkjhgfdsqwxcvbnAZERTYUIOPMLKJHGFDSQWXCVBN./012 3456789";
	gchar salt[] = "$1$XXXXXXXX$";

	int i;
	for (i = 3; i < 11; i++)
		salt[i] = s[rand() % (sizeof(s))];
	gchar *encoded = crypt(clear, salt);

	return encoded;
}

static void pwd_ok(GtkWidget * entry, gpointer data)
{
	PW *pwds = (PW *) data;
	gchar *t1, *t2;

	t1 = g_strdup(gtk_entry_get_text(GTK_ENTRY(pwds->entry1)));
	t2 = g_strdup(gtk_entry_get_text(GTK_ENTRY(pwds->entry2)));
	if (strlen(t1) && (strcmp(t1, t2) == 0)) {
		gtk_widget_set_sensitive(pwds->okbtn, TRUE);
		pwds->pwd = get_hash(t1);
	} else
		gtk_widget_set_sensitive(pwds->okbtn, FALSE);
	g_free(t1);
	g_free(t2);
}

/* returns the md5 hash */
gchar *get_lock_passwd()
{
	gchar *ret = NULL;
	GtkWidget *w, *label, *table;
	GtkWidget *cancelbtn;

	PW *pwds = g_new0(PW, 1);

	w = gtk_dialog_new();
    gtk_window_set_title (GTK_WINDOW (w), _("Password"));

	table = gtk_table_new(3, 2, FALSE);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(w)->vbox), table);
    gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG(w)->vbox), 6);

	label = gtk_label_new(NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span style=\"italic\">Select a password for the 'linutop' user:</span>"));
	gtk_table_attach(GTK_TABLE(table), label, 0, 2, 0, 1,
			 GTK_FILL, GTK_FILL, 2, 2);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

	pwds->entry1 = gtk_entry_new_with_max_length(16);
	gtk_entry_set_visibility(GTK_ENTRY(pwds->entry1), FALSE);
	gtk_table_attach(GTK_TABLE(table), pwds->entry1, 1, 2, 1, 2,
			 GTK_SHRINK, GTK_SHRINK, 2, 2);
	pwds->entry2 = gtk_entry_new_with_max_length(16);
	gtk_entry_set_visibility(GTK_ENTRY(pwds->entry2), FALSE);
	gtk_table_attach(GTK_TABLE(table), pwds->entry2, 1, 2, 2, 3,
			 GTK_SHRINK, GTK_SHRINK, 2, 2);
	label = gtk_label_new(_("Password: "));
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
			 GTK_SHRINK, GTK_SHRINK, 2, 2);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	label = gtk_label_new(_("Retype password: "));
	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
			 GTK_SHRINK, GTK_SHRINK, 2, 2);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

	cancelbtn = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
	pwds->okbtn = gtk_button_new_from_stock(GTK_STOCK_OK);
	gtk_widget_set_sensitive(pwds->okbtn, FALSE);
	gtk_dialog_add_action_widget(GTK_DIALOG(w), cancelbtn, CANCEL);
	gtk_dialog_add_action_widget(GTK_DIALOG(w), pwds->okbtn, OK);

	g_signal_connect(G_OBJECT(pwds->entry1), "changed",
			 G_CALLBACK(pwd_ok), pwds);
	g_signal_connect(G_OBJECT(pwds->entry2), "changed",
			 G_CALLBACK(pwd_ok), pwds);

	gtk_widget_show_all(w);
	int response = gtk_dialog_run(GTK_DIALOG(w));
	if (response == OK) {
		ret = g_strdup(pwds->pwd);
	}
	if (pwds->pwd)
    {
		memset(pwds->pwd, 0, sizeof(gchar) * strlen (pwds->pwd));
        g_free (pwds->pwd);
    }
	gtk_widget_destroy(w);
	g_free(pwds);

	return ret;
}
