/* language.c
 * Copyright (C) 2007-2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of the linutop config editor
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef __UTILS_H__
#define __UTILS_H__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <pwd.h>

#include <glib/gstdio.h>
#include <libxklavier/xklavier.h>
#include <libxfcegui4/libxfcegui4.h>

struct locale_item {
    const gchar *name;
    const gchar *code;
    const gchar *kbd;
    const gchar *tz;
};

extern const gchar *resolutions[11];
extern const struct locale_item locale_data[];

enum lid {
    LID_NONE = 0,
    LID_SUSPEND,
    LID_HALT,
    LID_NB
};

Display *dsp;
XklEngine *engine;
XklConfigRegistry *registry;
XklConfigRec *user_rec_config;

/* Prototypes */
void      cb_row_activated    (GtkWidget * dialog,
                               gpointer data);
GtkWidget *new_run_cmd_button (const gchar *icon, const gchar *txt,
                               const gchar *app, gboolean as_user);

gboolean  app_is_usable       (const gchar *app);
gchar    *get_home            (void);
gchar    *get_first_word      (const gchar *line);
void      run_user_cmd        (const char *user_cmd,
                               gboolean sn);
gchar    *get_locale_name     (const gchar *code);
gchar    *get_kbd_name        (const gchar *str);

void      apply_kbd_config    (const gchar * kbd);

void      get_from_language   (const gchar *locale,
                               gchar **kbd, gchar **tz);

gchar    *get_bashstyle_config (const gchar *conffile,
                                const gchar *key,
                                const gchar *fallback);
void      set_bashstyle_config (const gchar *conffile,
                                const gchar *key,
                                const gchar *value);

#endif /* __UTILS_H__ */
